#ifndef __LCD_H
#define __LCD_H

#include <vdr/thread.h>
#include <vdr/device.h>
#include <vdr/player.h>
#include <vdr/dvbplayer.h>
//#include "i18n.h"
#include "femonreceiver.h"

#define LCDPORT 13666
#define LCDHOST "localhost"
#define LCDMAXSTATES 6
#define LCDMAXSTATEBUF 6
#define LCDMAXWID 40
#define LCDMAXCARDS 4
#define LCDMAXFULLSTRING 1024

class cLcd : public cThread {

  public:
    cControl *replayDvbApi;	  
    cDevice *primaryDvbApi;	  
    enum ThreadStates {Menu=0, Title=1, Replay=2, Misc=3, Vol=4, SA=5};
    ThreadStates LastState[LCDMAXSTATEBUF];
    int LastStateP,LineMode;
    bool ToggleMode;
    struct StateData {
      char lcdbuffer[LCDMAXSTATES][4][LCDMAXWID+1];
      char lcdfullbuffer[LCDMAXSTATES][LCDMAXFULLSTRING];
      bool lcddirty[LCDMAXSTATES][4], muted, showvolume, newscroll;
      char replayString[LCDMAXFULLSTRING]; // needed for detection of the played media for Soundgraph-LCD
      ThreadStates State;
      unsigned int barx, bary, barl, volume;
      unsigned int CardStat[LCDMAXCARDS];
      int progressBarsValue;
      int outputValue;
    };
    cLcd();
    ~cLcd();
    bool Connect( char *host = LCDHOST , unsigned int port = LCDPORT );
    void Close();
    void Info();
    void SetTitle( const char *string);
//    void SetMain( unsigned int n, const char *string, bool menumode);
    void ClearSaBars();
    void determineReplayTypeIcons( const char* string );
    void SetMain( unsigned int n, const char *string, bool isConverted = false );
    void SetHelp( unsigned int n, const char *Red, const char *Green, const char *Yellow, const char *Blue);
    void SetStatus( const char *string);
    void SetWarning( const char *string);
    void ShowVolume(unsigned int vol, bool muted ); 
    void SetProgress(const char *begin=NULL, const char *end=NULL, int percent=0); 
    void SetLine(unsigned int n, unsigned int l, const char *string); 
    void SetLineC(unsigned int n, unsigned int l, const char *string); 
    void SetRunning(bool nownext, const char *string1="\0", const char *string2="\0", const char *string3="\0"); 
    void SummaryInit(const char *string);
    void SummaryUp();
    void SummaryDown();
    void SummaryDisplay();
    void SetCardStat(unsigned int card, unsigned int stat);
    void Clear(unsigned int n);
    void SetThreadState( ThreadStates newstate );
    void PopThreadState();
    void SetReplayDevice(cControl *DvbApi);
    void SetPrimaryDevice(cDevice *DvbApi);
    void ChannelSwitched(); //to propagate "ChannelSwitched"-Event from cLcdFeed to cLcd
    inline int volumeBar( int oldValue, double newValue ); // calculates values for the bars of the
    inline int progressBar( int oldValue, double newValue ); // Soundgraph LCD
    void audioTrackChanged();
  private:
    char *SummaryText;
    char replayTextTmp[1024];
    bool SAScroller;
    unsigned int SummaryTextL;
    unsigned int SummaryCurrent;
    bool connected;
    bool dimmed;
    bool channelSwitched;
    bool attachFemonReceiver;
    struct timeval dimm_time;
    FILE *f;
    struct StateData ThreadStateData;
    time_t LastProgress;
    cMutex CriticalArea;
    unsigned int sock, wid, hgt, cellwid, cellhgt;
    char StringBuffer[2*LCDMAXWID+1];
    cCharSetConv *conv;
    const char* Convert(const char *s);
    void BeginMutualExclusion();
    void EndMutualExclusion();    
    void Copy(char *to, const char *from, unsigned int max);
    void SetBuffer(unsigned int n,const char *l1=NULL,const char *l2=NULL,const char *l3=NULL,const char *l4=NULL);
    void Split(char *string, char *string1, char *string2);
    void Write(int line, const char *string);
    void GetTimeDateStat( char *string, unsigned int OutStateData[] );
    void Action(void);
    inline void backlightOn( /*struct timeval* */);
    inline bool isIMonLCD(void);
    int closing;
    cFemonReceiver *femonReceiver;
    struct timeval lastIcons;

};
// Radiotext
struct RadioTextService_v1_0 {
  int rds_info;
  int rds_pty;
  char *rds_text;
  char *rds_title;
  char *rds_artist;
  struct tm *title_start;    
};

// LcrData
struct LcrService_v1_0 {
  cString destination;
  cString price;
  cString pulse;
};

#define SPAN_PROVIDER_CHECK_ID 	"Span-ProviderCheck-v1.0"
#define SPAN_CLIENT_CHECK_ID 	"Span-ClientCheck-v1.0"
#define SPAN_SET_PCM_DATA_ID 	"Span-SetPcmData-v1.0"
#define SPAN_GET_BAR_HEIGHTS_ID "Span-GetBarHeights-v1.0"

//Span requests to collect possible providers / clients
struct Span_Provider_Check_1_0 {
	bool *isActive;
	bool *isRunning;
};
		
struct Span_Client_Check_1_0 {
	bool *isActive;
	bool *isRunning;
};
				
// Span data
struct Span_SetPcmData_1_0 {
	unsigned int length;	// the length of the PCM-data
	int *data;		// the PCM-Data as 32-bit int, however only the lower 16-bit are used
				// and you have to take care to hand in such data!
};

struct Span_GetBarHeights_v1_0 {
	unsigned int bands;			// number of bands to compute
	unsigned int *barHeights;		// the heights of the bars of the two channels combined
	unsigned int *barHeightsLeftChannel;	// the heights of the bars of the left channel
	unsigned int *barHeightsRightChannel;	// the heights of the bars of the right channel
	unsigned int *volumeLeftChannel;	// the volume of the left channels
	unsigned int *volumeRightChannel;	// the volume of the right channels
	unsigned int *volumeBothChannels;	// the combined volume of the two channels
	const char *name;			// name of the plugin that wants to get the data
						// (must be unique for each client!)
	unsigned int falloff;                    // bar falloff value
	unsigned int *barPeaksBothChannels;      // bar peaks of the two channels combined
	unsigned int *barPeaksLeftChannel;       // bar peaks of the left channel
	unsigned int *barPeaksRightChannel;      // bar peaks of the right channel
};
#endif //__LCD_H
